"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRelationsForEvent = getRelationsForEvent;
const matrix_protection_suite_1 = require("matrix-protection-suite");
async function getRelationsForEvent(client, decoder, roomID, eventID, options) {
    let url = `/_matrix/client/v1/rooms/${encodeURIComponent(roomID)}/relations/${encodeURIComponent(eventID)}`;
    if (options.relationType) {
        url += `/${options.relationType}`;
    }
    if (options.eventType) {
        url += `/${options.eventType}`;
    }
    const errorContext = `relations for event ${eventID} in room ${roomID}`;
    return await client
        .doRequest('GET', url, {
        dir: options.direction === 'forwards' ? 'f' : 'b',
        // for some reason the bot-sdk treats undefined properties like { from: undefined } as &from=
        ...(options.from !== undefined ? { from: options.from } : {}),
        ...(options.itemLimitPerPage !== undefined
            ? { limit: options.itemLimitPerPage }
            : {}),
        ...(options.to !== undefined ? { to: options.to } : {}),
    })
        .then((rawChunk) => {
        const chunkResult = matrix_protection_suite_1.Value.Decode(matrix_protection_suite_1.PaginatedResponse, rawChunk);
        if ((0, matrix_protection_suite_1.isError)(chunkResult)) {
            return chunkResult.elaborate(`Could not decode the chunk response while getting ${errorContext}`);
        }
        const chunkies = [];
        for (const item of chunkResult.ok.chunk) {
            const decodeResult = decoder.decodeEvent(item);
            if ((0, matrix_protection_suite_1.isError)(decodeResult)) {
                return decodeResult.elaborate(`Was unexpectadly unable to decode an event while getting relations`);
            }
            chunkies.push(decodeResult.ok);
        }
        return (0, matrix_protection_suite_1.Ok)({
            next_batch: chunkResult.ok.next_batch,
            prev_batch: chunkResult.ok.prev_batch,
            chunk: chunkies,
        });
    }, (exception) => matrix_protection_suite_1.ActionException.Result(`Could not request ${errorContext}`, {
        exception,
        exceptionKind: matrix_protection_suite_1.ActionExceptionKind.Unknown,
    }));
}
//# sourceMappingURL=PaginationAPIs.js.map