import { StaticDecode } from '@sinclair/typebox';
import { ActionError, ActionResult } from '../Interface/Action';
declare const PaginationTokenSecret: unique symbol;
export type StringPaginationToken = string & {
    [PaginationTokenSecret]: true;
};
export declare const StringPaginationToken: import("@sinclair/typebox").TUnsafe<StringPaginationToken>;
export interface ChunkPage<ChunkItem> {
    getNextPage(): Promise<ActionResult<ChunkPage<ChunkItem>, PaginationError>>;
    chunk: ChunkItem[];
    nextToken: StringPaginationToken | undefined;
    previousToken: StringPaginationToken | undefined;
    hasNext(): boolean;
    hasPrevious(): boolean;
    isFirstPage(): boolean;
}
export interface PaginationOptions {
    direction?: 'forwards' | 'backwards';
    from?: StringPaginationToken | undefined;
    itemLimitPerPage?: number | undefined;
    to?: StringPaginationToken | undefined;
}
export type StreamPaginationForEachCB<ChunkItem> = (item: ChunkItem) => void;
export type StreamPaginationTestStopCB<ChunkItem> = (item: ChunkItem) => boolean;
export interface StreamPaginationOptions<ChunkItem> extends PaginationOptions {
    totalItemLimit?: number;
    forEachCB?: StreamPaginationForEachCB<ChunkItem>;
    testStopCB?: StreamPaginationTestStopCB<ChunkItem>;
}
export interface PaginationError extends ActionError {
    nextToken?: StringPaginationToken;
    previousToken?: StringPaginationToken;
}
export declare function doPagination<ChunkItem>(startingPage: ChunkPage<ChunkItem>, options: StreamPaginationOptions<ChunkItem>): Promise<ActionResult<void, PaginationError>>;
export type StandardChunkPageGetter<ChunkItem> = (options: PaginationOptions) => Promise<ActionResult<PaginatedResponse<ChunkItem>, PaginationError>>;
export declare class StandardChunkPage<ChunkItem> implements ChunkPage<ChunkItem> {
    #private;
    readonly chunk: ChunkItem[];
    private readonly chunkGetter;
    private readonly paginationOptions;
    readonly nextToken: StringPaginationToken | undefined;
    readonly previousToken: StringPaginationToken | undefined;
    static createFirstPage<ChunkItem>(chunkGetter: StandardChunkPageGetter<ChunkItem>, paginationOptions: StreamPaginationOptions<ChunkItem>): StandardChunkPage<ChunkItem>;
    private constructor();
    isFirstPage(): boolean;
    hasNext(): boolean;
    hasPrevious(): boolean;
    getNextPage(): Promise<ActionResult<ChunkPage<ChunkItem>, PaginationError>>;
}
export declare const PaginatedResponse: import("@sinclair/typebox").TObject<{
    chunk: import("@sinclair/typebox").TArray<import("@sinclair/typebox").TUnknown>;
    prev_batch: import("@sinclair/typebox").TOptional<import("@sinclair/typebox").TUnsafe<StringPaginationToken>>;
    next_batch: import("@sinclair/typebox").TOptional<import("@sinclair/typebox").TUnsafe<StringPaginationToken>>;
}>;
export type PaginatedResponse<ChunkItem = unknown> = Omit<StaticDecode<typeof PaginatedResponse>, 'chunk'> & {
    chunk: ChunkItem[];
};
export {};
//# sourceMappingURL=Pagination.d.ts.map