"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _StandardChunkPage_isFirstPage;
Object.defineProperty(exports, "__esModule", { value: true });
exports.PaginatedResponse = exports.StandardChunkPage = exports.StringPaginationToken = void 0;
exports.doPagination = doPagination;
// SPDX-FileCopyrightText: 2024 Gnuxie <Gnuxie@protonmail.com>
//
// SPDX-License-Identifier: AFL-3.0
const typebox_1 = require("@sinclair/typebox");
const Action_1 = require("../Interface/Action");
const PaginationTokenSecret = Symbol('PaginationTokenSecret');
typebox_1.FormatRegistry.Set('StringPaginationToken', (thing) => typeof thing === 'string');
exports.StringPaginationToken = typebox_1.Type.Unsafe(typebox_1.Type.String({ format: 'StringPaginationToken' }));
async function doPagination(startingPage, options) {
    var _a, _b, _c;
    const isUnderLimit = (count) => options.totalItemLimit === undefined || count < options.totalItemLimit;
    let itemCount = 0;
    let currentPage = startingPage;
    while (true) {
        let isMarkedAsStop = false;
        for (const item of currentPage.chunk) {
            isMarkedAsStop =
                !isUnderLimit(itemCount) || ((_b = (_a = options.testStopCB) === null || _a === void 0 ? void 0 : _a.call(options, item)) !== null && _b !== void 0 ? _b : false);
            if (isMarkedAsStop) {
                break;
            }
            (_c = options.forEachCB) === null || _c === void 0 ? void 0 : _c.call(options, item);
            itemCount++;
        }
        if (isMarkedAsStop) {
            break;
        }
        if (!currentPage.hasNext() && !currentPage.isFirstPage()) {
            break; // no more items.
        }
        const nextPageResult = await currentPage.getNextPage();
        if ((0, Action_1.isError)(nextPageResult)) {
            return nextPageResult;
        }
        else {
            currentPage = nextPageResult.ok;
        }
    }
    return (0, Action_1.Ok)(undefined);
}
class StandardChunkPage {
    static createFirstPage(chunkGetter, paginationOptions) {
        return new StandardChunkPage([], chunkGetter, true, {
            nextToken: paginationOptions.from,
        }, paginationOptions);
    }
    constructor(chunk, chunkGetter, isFirstPage, { nextToken, previousToken, }, paginationOptions) {
        this.chunk = chunk;
        this.chunkGetter = chunkGetter;
        this.paginationOptions = paginationOptions;
        _StandardChunkPage_isFirstPage.set(this, void 0);
        __classPrivateFieldSet(this, _StandardChunkPage_isFirstPage, isFirstPage, "f");
        this.nextToken = nextToken;
        this.previousToken = previousToken;
    }
    isFirstPage() {
        return __classPrivateFieldGet(this, _StandardChunkPage_isFirstPage, "f");
    }
    hasNext() {
        return this.nextToken !== undefined;
    }
    hasPrevious() {
        return this.previousToken !== undefined;
    }
    async getNextPage() {
        if (!this.hasNext() && !this.isFirstPage()) {
            throw new TypeError(`You're not checking whether the page hasNext() before iterating further`);
        }
        const nextChunk = await this.chunkGetter({
            ...this.paginationOptions,
            from: this.nextToken,
        });
        if ((0, Action_1.isError)(nextChunk)) {
            return nextChunk;
        }
        return (0, Action_1.Ok)(new StandardChunkPage(nextChunk.ok.chunk, this.chunkGetter, false, {
            nextToken: nextChunk.ok.next_batch,
            previousToken: nextChunk.ok.prev_batch,
        }, this.paginationOptions));
    }
}
exports.StandardChunkPage = StandardChunkPage;
_StandardChunkPage_isFirstPage = new WeakMap();
exports.PaginatedResponse = typebox_1.Type.Object({
    chunk: typebox_1.Type.Array(typebox_1.Type.Unknown()),
    prev_batch: typebox_1.Type.Optional(exports.StringPaginationToken),
    next_batch: typebox_1.Type.Optional(exports.StringPaginationToken),
});
//# sourceMappingURL=Pagination.js.map