"use strict";
// SPDX-FileCopyrightText: 2025 Gnuxie <Gnuxie@protonmail.com>
//
// SPDX-License-Identifier: AFL-3.0
Object.defineProperty(exports, "__esModule", { value: true });
exports.RoomsSetBehaviour = void 0;
const matrix_protection_suite_1 = require("matrix-protection-suite");
const typescript_result_1 = require("@gnuxie/typescript-result");
const ProtectJoinedRooms_1 = require("./ProtectJoinedRooms");
const UnprotectPartedRooms_1 = require("./UnprotectPartedRooms");
class RoomsSetBehaviour extends matrix_protection_suite_1.AbstractProtection {
    constructor(description, capabilities, protectedRoomsSet, draupnir) {
        super(description, capabilities, protectedRoomsSet, {});
        this.draupnir = draupnir;
        this.protectJoinedRooms = new ProtectJoinedRooms_1.ProtectedJoinedRooms(this.draupnir.clientUserID, this.draupnir.managementRoomID, this.protectedRoomsSet, this.draupnir.clientRooms, this.draupnir.clientPlatform.toRoomMessageSender());
        this.unprotectedPartedRooms = new UnprotectPartedRooms_1.UnprotectPartedRooms(this.draupnir.clientUserID, this.draupnir.managementRoomID, this.protectedRoomsSet.protectedRoomsManager, this.draupnir.clientPlatform.toRoomMessageSender());
        if (this.draupnir.config.protectAllJoinedRooms) {
            void this.protectJoinedRooms.syncProtectedRooms();
        }
    }
    handleMembershipChange(_revision, changes) {
        if (this.draupnir.config.protectAllJoinedRooms) {
            this.protectJoinedRooms.handleMembershipChange(changes);
        }
        for (const change of changes) {
            this.unprotectedPartedRooms.handleMembershipChange(change);
        }
        return Promise.resolve((0, typescript_result_1.Ok)(undefined));
    }
    handleExternalMembership(roomID, event) {
        if (this.draupnir.config.protectAllJoinedRooms) {
            this.protectJoinedRooms.handleExternalMembership(roomID, event);
        }
    }
}
exports.RoomsSetBehaviour = RoomsSetBehaviour;
(0, matrix_protection_suite_1.describeProtection)({
    name: RoomsSetBehaviour.name,
    description: "Unprotects parted rooms and update the list of protected rooms.",
    capabilityInterfaces: {},
    defaultCapabilities: {},
    async factory(description, protectedRoomsSet, draupnir, capabilities, _settings) {
        return (0, typescript_result_1.Ok)(new RoomsSetBehaviour(description, capabilities, protectedRoomsSet, draupnir));
    },
});
//# sourceMappingURL=RoomsSetBehaviourProtection.js.map